import Model

extension Board {
    public func display(winCells: [Coords]) -> String {
        var str = String()
        
        let rows = Self.numbers(size: self.rows)
        let columns = Self.numbers(size: self.columns)
        
        for row in 0..<self.rows {
            if row == 0 {
                str += String(repeating: " ", count: rows.size)
                str += columns.nums.joined()
                str += "\n"
            }
            
            str += rows.nums[row]
            
            for col in 0..<self.columns {
                let coord = Coords(col, row)
                
                let char = switch (self[coord]?.type, winCells.contains(coord)) {
                case (.A, false): "🔴"
                case (.A, true): "🟥"
                case (.B, false): "🟡"
                case (.B, true): "🟨"
                default: "⚪"
                }
                
                str += String(repeating: " ", count: columns.size - 2) + char
            }
            
            str += "\n"
        }
        
        return str
    }
    
    private static func numbers(size: Int) -> (nums: [String], size: Int) {
        let elems = Array((1...size).map { String($0) })
        let size = max(2, elems.lazy.map { $0.count }.max() ?? 0)
        return (nums: elems.map { String(repeating: " ", count: size - $0.count) + $0 }, size: size )
    }
}

extension Board: CustomStringConvertible, CustomDebugStringConvertible {
    public var description: String {
        self.display(winCells: [])
    }
    
    public var debugDescription: String {
        return "Board[\(self.columns)x\(self.rows), \(self.countPieces { p in p.type == .A }) A and \(self.countPieces { p in p.type == .B }) B pieces]"
    }
}


