public protocol Rules: Equatable {
    var type: RulesTypes { get }

    func createBoard() -> Board

    func isValid(board: Board) -> Bool

    func isValid(board: Board, move: Move) -> Bool

    func validMoves(board: Board) -> [Move]

    func validMoves(board: Board, for_player player: Player) -> [Move.Action]

    func gameState(board: Board, last_turn: Player?) -> GameState
}

public struct RulesUtils {
    public static func equals(lhs: any Rules, rhs: any Rules) -> Bool {
        let t1 = lhs.type
        let t2 = rhs.type
        
        if t1 != t2 { return false }
        
        return switch t1 {
        case .FourInARow:
            (lhs as! FourInARowRules) == (rhs as! FourInARowRules)
        case .TicTacToe:
            (lhs as! TicTacToeRules) == (rhs as! TicTacToeRules)
        }
    }
}

public enum GameState: Equatable {
    case Playing(turn: Player)
    case Win(winner: Player, board: Board, cells: [Coords])
    case Draw
    
    /// In rare cases, the state may be invalid because the game was incorrectly reconstructed (serialization, ...)
    case Invalid
}

public enum RulesTypes: CaseIterable {
    case FourInARow, TicTacToe
}
